@@ -1,7 +1,6 @@
 #ifndef RUBY_H                                       /*-*-C++-*-vi:se ft=cpp:*/
 #define RUBY_H 1
 /**
- * @file
  * @author     $Author$
  * @date       Sun 10 12:06:15 Jun JST 2007
  * @copyright  2007-2008 Yukihiro Matsumoto
@@ -14,6 +13,7 @@
 #define HAVE_RUBY_DEBUG_H       1
 #define HAVE_RUBY_DEFINES_H     1
 #define HAVE_RUBY_ENCODING_H    1
+#define HAVE_RUBY_FIBER_SCHEDULER_H 1
 #define HAVE_RUBY_INTERN_H      1
 #define HAVE_RUBY_IO_H          1
 #define HAVE_RUBY_MEMORY_VIEW_H 1
