@@ -121,7 +121,7 @@
 
 module Benchmark
 
-  VERSION = "0.4.0"
+  VERSION = "0.3.0"
 
   BENCHMARK_VERSION = "2002-04-25" # :nodoc:
 
@@ -173,15 +173,9 @@ module Benchmark
     label_width ||= 0
     label_width += 1
     format ||= FORMAT
+    print ' '*label_width + caption unless caption.empty?
     report = Report.new(label_width, format)
     results = yield(report)
-
-    print " " * report.width + caption unless caption.empty?
-    report.list.each { |i|
-      print i.label.to_s.ljust(report.width)
-      print i.format(report.format, *format)
-    }
-
     Array === results and results.grep(Tms).each {|t|
       print((labels.shift || t.label || "").ljust(label_width), t.format(format))
     }
@@ -313,10 +307,6 @@ module Benchmark
 
   #
   # Returns the elapsed real time used to execute the given block.
-  # The unit of time is seconds.
-  #
-  #       Benchmark.realtime { "a" * 1_000_000_000 }
-  #       #=> 0.5098029999935534
   #
   def realtime # :yield:
     r0 = Process.clock_gettime(Process::CLOCK_MONOTONIC)
@@ -386,16 +376,16 @@ module Benchmark
     # formatting rules.
     #
     def item(label = "", *format, &blk) # :yield:
-      w = label.to_s.length
-      @width = w if @width < w
+      print label.to_s.ljust(@width)
       @list << res = Benchmark.measure(label, &blk)
+      print res.format(@format, *format)
       res
     end
 
     alias report item
 
     # An array of Benchmark::Tms objects representing each item.
-    attr_reader :width, :format, :list
+    attr_reader :list
   end
 
 
