@@ -26,6 +26,12 @@ module Gem::BUNDLED_GEMS
     "resolv-replace" => "3.4.0",
     "rinda" => "3.4.0",
     "syslog" => "3.4.0",
+    "ostruct" => "3.5.0",
+    "pstore" => "3.5.0",
+    "rdoc" => "3.5.0",
+    "win32ole" => "3.5.0",
+    "fiddle" => "3.5.0",
+    "logger" => "3.5.0",
   }.freeze
 
   SINCE_FAST_PATH = SINCE.transform_keys { |g| g.sub(/\A.*\-/, "") }.freeze
