@@ -12,15 +12,13 @@ module Bundler
     require_relative "resolver/candidate"
     require_relative "resolver/incompatibility"
     require_relative "resolver/root"
-    require_relative "resolver/strategy"
 
     include GemHelpers
 
-    def initialize(base, gem_version_promoter, most_specific_locked_platform = nil)
+    def initialize(base, gem_version_promoter)
       @source_requirements = base.source_requirements
       @base = base
       @gem_version_promoter = gem_version_promoter
-      @most_specific_locked_platform = most_specific_locked_platform
     end
 
     def start
@@ -79,16 +77,16 @@ module Bundler
     end
 
     def solve_versions(root:, logger:)
-      solver = PubGrub::VersionSolver.new(source: self, root: root, strategy: Strategy.new(self), logger: logger)
+      solver = PubGrub::VersionSolver.new(source: self, root: root, logger: logger)
       result = solver.solve
-      resolved_specs = result.flat_map {|package, version| version.to_specs(package, @most_specific_locked_platform) }
-      SpecSet.new(resolved_specs).specs_with_additional_variants_from(@base.locked_specs)
+      resolved_specs = result.map {|package, version| version.to_specs(package) }.flatten
+      resolved_specs |= @base.specs_compatible_with(SpecSet.new(resolved_specs))
     rescue PubGrub::SolveFailure => e
       incompatibility = e.incompatibility
 
-      names_to_unlock, names_to_allow_prereleases_for, names_to_allow_remote_specs_for, extended_explanation = find_names_to_relax(incompatibility)
+      names_to_unlock, names_to_allow_prereleases_for, extended_explanation = find_names_to_relax(incompatibility)
 
-      names_to_relax = names_to_unlock + names_to_allow_prereleases_for + names_to_allow_remote_specs_for
+      names_to_relax = names_to_unlock + names_to_allow_prereleases_for
 
       if names_to_relax.any?
         if names_to_unlock.any?
@@ -98,17 +96,11 @@ module Bundler
         end
 
         if names_to_allow_prereleases_for.any?
-          Bundler.ui.debug "Found conflicts with dependencies with prereleases. Will retry considering prereleases for #{names_to_allow_prereleases_for.join(", ")}...", true
+          Bundler.ui.debug "Found conflicts with dependencies with prereleases. Will retrying considering prereleases for #{names_to_allow_prereleases_for.join(", ")}...", true
 
           @base.include_prereleases(names_to_allow_prereleases_for)
         end
 
-        if names_to_allow_remote_specs_for.any?
-          Bundler.ui.debug "Found conflicts with local versions of #{names_to_allow_remote_specs_for.join(", ")}. Will retry considering remote versions...", true
-
-          @base.include_remote_specs(names_to_allow_remote_specs_for)
-        end
-
         root, logger = setup_solver
 
         Bundler.ui.debug "Retrying resolution...", true
@@ -128,7 +120,6 @@ module Bundler
     def find_names_to_relax(incompatibility)
       names_to_unlock = []
       names_to_allow_prereleases_for = []
-      names_to_allow_remote_specs_for = []
       extended_explanation = nil
 
       while incompatibility.conflict?
@@ -143,8 +134,6 @@ module Bundler
             names_to_unlock << name
           elsif package.ignores_prereleases? && @all_specs[name].any? {|s| s.version.prerelease? }
             names_to_allow_prereleases_for << name
-          elsif package.prefer_local? && @all_specs[name].any? {|s| !s.is_a?(StubSpecification) }
-            names_to_allow_remote_specs_for << name
           end
 
           no_versions_incompat = [cause.incompatibility, cause.satisfier].find {|incompat| incompat.cause.is_a?(PubGrub::Incompatibility::NoVersions) }
@@ -154,7 +143,7 @@ module Bundler
         end
       end
 
-      [names_to_unlock.uniq, names_to_allow_prereleases_for.uniq, names_to_allow_remote_specs_for.uniq, extended_explanation]
+      [names_to_unlock.uniq, names_to_allow_prereleases_for.uniq, extended_explanation]
     end
 
     def parse_dependency(package, dependency)
@@ -168,7 +157,15 @@ module Bundler
     end
 
     def versions_for(package, range=VersionRange.any)
-      range.select_versions(@sorted_versions[package])
+      versions = select_sorted_versions(package, range)
+
+      # Conditional avoids (among other things) calling
+      # sort_versions_by_preferred with the root package
+      if versions.size > 1
+        sort_versions_by_preferred(package, versions)
+      else
+        versions
+      end
     end
 
     def no_versions_incompatibility_for(package, unsatisfied_term)
@@ -230,7 +227,7 @@ module Bundler
             sorted_versions[high]
           end
 
-        range = PubGrub::VersionRange.new(min: low, max: high, include_min: !low.nil?)
+        range = PubGrub::VersionRange.new(min: low, max: high, include_min: true)
 
         self_constraint = PubGrub::VersionConstraint.new(package, range: range)
 
@@ -247,7 +244,7 @@ module Bundler
 
     def all_versions_for(package)
       name = package.name
-      results = (@base[name] + filter_specs(@all_specs[name], package)).uniq {|spec| [spec.version.hash, spec.platform] }
+      results = (@base[name] + filter_prereleases(@all_specs[name], package)).uniq {|spec| [spec.version.hash, spec.platform] }
 
       if name == "bundler" && !bundler_pinned_to_current_version?
         bundler_spec = Gem.loaded_specs["bundler"]
@@ -277,8 +274,8 @@ module Bundler
         ruby_group = Resolver::SpecGroup.new(ruby_specs)
 
         unless ruby_group.empty?
-          platform_specs.each do |s|
-            ruby_group.merge(Resolver::SpecGroup.new(s))
+          platform_specs.each do |specs|
+            ruby_group.merge(Resolver::SpecGroup.new(specs))
           end
 
           groups << Resolver::Candidate.new(version, group: ruby_group, priority: -1)
@@ -312,16 +309,6 @@ module Bundler
       "Gemfile"
     end
 
-    def raise_incomplete!(incomplete_specs)
-      raise_not_found!(@base.get_package(incomplete_specs.first.name))
-    end
-
-    def sort_versions_by_preferred(package, versions)
-      @gem_version_promoter.sort_versions(package, versions)
-    end
-
-    private
-
     def raise_not_found!(package)
       name = package.name
       source = source_for(name)
@@ -358,6 +345,8 @@ module Bundler
       raise GemNotFound, message
     end
 
+    private
+
     def filtered_versions_for(package)
       @gem_version_promoter.filter_versions(package, @all_versions[package])
     end
@@ -379,31 +368,12 @@ module Bundler
       end
     end
 
-    def filter_specs(specs, package)
-      filter_remote_specs(filter_prereleases(specs, package), package)
-    end
-
     def filter_prereleases(specs, package)
       return specs unless package.ignores_prereleases? && specs.size > 1
 
       specs.reject {|s| s.version.prerelease? }
     end
 
-    def filter_remote_specs(specs, package)
-      if package.prefer_local?
-        local_specs = specs.select {|s| s.is_a?(StubSpecification) }
-
-        if local_specs.empty?
-          package.consider_remote_versions!
-          specs
-        else
-          local_specs
-        end
-      else
-        specs
-      end
-    end
-
     # Ignore versions that depend on themselves incorrectly
     def filter_invalid_self_dependencies(specs, name)
       specs.reject do |s|
@@ -415,6 +385,10 @@ module Bundler
       requirement.satisfied_by?(spec.version) || spec.source.is_a?(Source::Gemspec)
     end
 
+    def sort_versions_by_preferred(package, versions)
+      @gem_version_promoter.sort_versions(package, versions)
+    end
+
     def repository_for(package)
       source_for(package.name)
     end
@@ -424,21 +398,18 @@ module Bundler
     end
 
     def prepare_dependencies(requirements, packages)
-      to_dependency_hash(requirements, packages).filter_map do |dep_package, dep_constraint|
+      to_dependency_hash(requirements, packages).map do |dep_package, dep_constraint|
         name = dep_package.name
 
         next [dep_package, dep_constraint] if name == "bundler"
 
         dep_range = dep_constraint.range
-        versions = versions_for(dep_package, dep_range)
-        if versions.empty?
-          if dep_package.ignores_prereleases? || dep_package.prefer_local?
-            @all_versions.delete(dep_package)
-            @sorted_versions.delete(dep_package)
-          end
-          dep_package.consider_prereleases! if dep_package.ignores_prereleases?
-          dep_package.consider_remote_versions! if dep_package.prefer_local?
-          versions = versions_for(dep_package, dep_range)
+        versions = select_sorted_versions(dep_package, dep_range)
+        if versions.empty? && dep_package.ignores_prereleases?
+          @all_versions.delete(dep_package)
+          @sorted_versions.delete(dep_package)
+          dep_package.consider_prereleases!
+          versions = select_sorted_versions(dep_package, dep_range)
         end
 
         if versions.empty? && select_all_versions(dep_package, dep_range).any?
@@ -450,7 +421,11 @@ module Bundler
         next unless dep_package.current_platform?
 
         raise_not_found!(dep_package)
-      end.to_h
+      end.compact.to_h
+    end
+
+    def select_sorted_versions(package, range)
+      range.select_versions(@sorted_versions[package])
     end
 
     def select_all_versions(package, range)
