@@ -16,7 +16,7 @@ module Bundler
         def initialize(command)
           msg = String.new
           msg << "Bundler is trying to run `#{command}` at runtime. You probably need to run `bundle install`. However, "
-          msg << "this error message could probably be more useful. Please submit a ticket at https://github.com/ruby/rubygems/issues/new?labels=Bundler&template=bundler-related-issue.md "
+          msg << "this error message could probably be more useful. Please submit a ticket at https://github.com/rubygems/rubygems/issues/new?labels=Bundler&template=bundler-related-issue.md "
           msg << "with steps to reproduce as well as the following\n\nCALLER: #{caller.join("\n")}"
           super msg
         end
@@ -121,7 +121,7 @@ module Bundler
                 FileUtils.rm_rf(p)
               end
               git "clone", "--no-checkout", "--quiet", path.to_s, destination.to_s
-              File.chmod((File.stat(destination).mode | 0o777) & ~File.umask, destination)
+              File.chmod(((File.stat(destination).mode | 0o777) & ~File.umask), destination)
             rescue Errno::EEXIST => e
               file_path = e.message[%r{.*?((?:[a-zA-Z]:)?/.*)}, 1]
               raise GitError, "Bundler could not install a gem because it needs to " \
@@ -147,12 +147,6 @@ module Bundler
           end
         end
 
-        def installed_to?(destination)
-          # if copy_to is interrupted, it may leave a partially installed directory that
-          # contains .git but no other files -- consider this not to be installed
-          Dir.exist?(destination) && (Dir.children(destination) - [".git"]).any?
-        end
-
         private
 
         def git_remote_fetch(args)
@@ -185,8 +179,7 @@ module Bundler
             _, err, status = capture(command, nil)
             return extra_ref if status.success?
 
-            if err.include?("Could not find remote branch") || # git up to 2.49
-               err.include?("Remote branch #{branch_option} not found") # git 2.49 or higher
+            if err.include?("Could not find remote branch")
               raise MissingGitRevisionError.new(command_with_no_credentials, nil, explicit_ref, credential_filtered_uri)
             else
               idx = command.index("--depth")
@@ -263,7 +256,7 @@ module Bundler
         end
 
         def not_pinned?
-          branch_option || ref.nil?
+          branch || tag || ref.nil?
         end
 
         def pinned_to_full_sha?
@@ -305,8 +298,8 @@ module Bundler
         end
 
         def has_revision_cached?
-          return unless commit && path.exist?
-          git("cat-file", "-e", commit, dir: path)
+          return unless @revision && path.exist?
+          git("cat-file", "-e", @revision, dir: path)
           true
         rescue GitError
           false
@@ -339,6 +332,8 @@ module Bundler
             config_auth = Bundler.settings[remote.to_s] || Bundler.settings[remote.host]
             remote.userinfo ||= config_auth
             remote.to_s
+          elsif File.exist?(uri)
+            "file://#{uri}"
           else
             uri.to_s
           end
@@ -408,7 +403,11 @@ module Bundler
         def capture3_args_for(cmd, dir)
           return ["git", *cmd] unless dir
 
-          ["git", "-C", dir.to_s, *cmd]
+          if Bundler.feature_flag.bundler_3_mode? || supports_minus_c?
+            ["git", "-C", dir.to_s, *cmd]
+          else
+            ["git", *cmd, { chdir: dir.to_s }]
+          end
         end
 
         def extra_clone_args
@@ -423,7 +422,7 @@ module Bundler
           # anyways.
           return args if @revision
 
-          args += ["--branch", branch_option] if branch_option
+          args += ["--branch", branch || tag] if branch || tag
           args
         end
 
@@ -439,14 +438,14 @@ module Bundler
           extra_args
         end
 
-        def branch_option
-          branch || tag
-        end
-
         def full_clone?
           depth.nil?
         end
 
+        def supports_minus_c?
+          @supports_minus_c ||= Gem::Version.new(version) >= Gem::Version.new("1.8.5")
+        end
+
         def needs_allow_any_sha1_in_want?
           @needs_allow_any_sha1_in_want ||= Gem::Version.new(version) <= Gem::Version.new("2.13.7")
         end
