@@ -9,14 +9,15 @@
 require "rbconfig"
 
 module Gem
-  VERSION = "4.0.3"
+  VERSION = "3.5.16"
 end
 
+# Must be first since it unloads the prelude from 1.9.2
+require_relative "rubygems/compatibility"
+
 require_relative "rubygems/defaults"
 require_relative "rubygems/deprecate"
 require_relative "rubygems/errors"
-require_relative "rubygems/target_rbconfig"
-require_relative "rubygems/win_platform"
 
 ##
 # RubyGems is the Ruby standard for publishing and managing third party
@@ -37,7 +38,7 @@ require_relative "rubygems/win_platform"
 # Further RubyGems documentation can be found at:
 #
 # * {RubyGems Guides}[https://guides.rubygems.org]
-# * {RubyGems API}[https://www.rubydoc.info/github/ruby/rubygems] (also available from
+# * {RubyGems API}[https://www.rubydoc.info/github/rubygems/rubygems] (also available from
 #   <tt>gem server</tt>)
 #
 # == RubyGems Plugins
@@ -69,7 +70,7 @@ require_relative "rubygems/win_platform"
 # == Bugs
 #
 # You can submit bugs to the
-# {RubyGems bug tracker}[https://github.com/ruby/rubygems/issues]
+# {RubyGems bug tracker}[https://github.com/rubygems/rubygems/issues]
 # on GitHub
 #
 # == Credits
@@ -105,7 +106,7 @@ require_relative "rubygems/win_platform"
 #
 # == License
 #
-# See {LICENSE.txt}[https://github.com/ruby/rubygems/blob/master/LICENSE.txt] for permissions.
+# See {LICENSE.txt}[rdoc-ref:lib/rubygems/LICENSE.txt] for permissions.
 #
 # Thanks!
 #
@@ -114,6 +115,18 @@ require_relative "rubygems/win_platform"
 module Gem
   RUBYGEMS_DIR = __dir__
 
+  ##
+  # An Array of Regexps that match windows Ruby platforms.
+
+  WIN_PATTERNS = [
+    /bccwin/i,
+    /cygwin/i,
+    /djgpp/i,
+    /mingw/i,
+    /mswin/i,
+    /wince/i,
+  ].freeze
+
   GEM_DEP_FILES = %w[
     gem.deps.rb
     gems.rb
@@ -142,12 +155,7 @@ module Gem
     specifications/default
   ].freeze
 
-  ##
-  # The default value for SOURCE_DATE_EPOCH if not specified.
-  # We want a date after 1980-01-01, to prevent issues with Zip files.
-  # This particular timestamp is for 1980-01-02 00:00:00 GMT.
-
-  DEFAULT_SOURCE_DATE_EPOCH = 315_619_200
+  @@win_platform = nil
 
   @configuration = nil
   @gemdeps = nil
@@ -171,8 +179,6 @@ module Gem
 
   @discover_gems_on_require = true
 
-  @target_rbconfig = nil
-
   ##
   # Try to activate a gem containing +path+. Returns true if
   # activation succeeded or wasn't needed because it was already
@@ -211,7 +217,7 @@ module Gem
     finish_resolve rs
   end
 
-  def self.finish_resolve(request_set = Gem::RequestSet.new)
+  def self.finish_resolve(request_set=Gem::RequestSet.new)
     request_set.import Gem::Specification.unresolved_deps.values
     request_set.import Gem.loaded_specs.values.map {|s| Gem::Dependency.new(s.name, s.version) }
 
@@ -233,16 +239,6 @@ module Gem
     find_spec_for_exe(name, exec_name, requirements).bin_file exec_name
   end
 
-  def self.find_and_activate_spec_for_exe(name, exec_name, requirements)
-    spec = find_spec_for_exe name, exec_name, requirements
-    Gem::LOADED_SPECS_MUTEX.synchronize do
-      spec.activate
-      finish_resolve
-    end
-    spec
-  end
-  private_class_method :find_and_activate_spec_for_exe
-
   def self.find_spec_for_exe(name, exec_name, requirements)
     raise ArgumentError, "you must supply exec_name" unless exec_name
 
@@ -267,42 +263,6 @@ module Gem
   end
   private_class_method :find_spec_for_exe
 
-  ##
-  # Find and load the full path to the executable for gem +name+.  If the
-  # +exec_name+ is not given, an exception will be raised, otherwise the
-  # specified executable's path is returned.  +requirements+ allows
-  # you to specify specific gem versions.
-  #
-  # A side effect of this method is that it will activate the gem that
-  # contains the executable.
-  #
-  # This method should *only* be used in bin stub files.
-
-  def self.activate_and_load_bin_path(name, exec_name = nil, *requirements)
-    spec = find_and_activate_spec_for_exe name, exec_name, requirements
-
-    if spec.name == "bundler"
-      # Old versions of Bundler need a workaround to support nested `bundle
-      # exec` invocations by overriding `Gem.activate_bin_path`. However,
-      # RubyGems now uses this new `Gem.activate_and_load_bin_path` helper in
-      # binstubs, which is of course not overridden in Bundler since it didn't
-      # exist at the time. So, include the override here to workaround that.
-      load ENV["BUNDLE_BIN_PATH"] if ENV["BUNDLE_BIN_PATH"] && spec.version <= Gem::Version.create("2.5.22")
-
-      # Make sure there's no version of Bundler in `$LOAD_PATH` that's different
-      # from the version we just activated. If that was the case (it happens
-      # when testing Bundler from ruby/ruby), we would load Bundler extensions
-      # to RubyGems from the copy in `$LOAD_PATH` but then load the binstub from
-      # an installed copy, causing those copies to be mixed and yet more
-      # redefinition warnings.
-      #
-      require_path = $LOAD_PATH.resolve_feature_path("bundler").last.delete_suffix("/bundler.rb")
-      Gem.load_bundler_extensions(spec.version) if spec.full_require_paths.include?(require_path)
-    end
-
-    load spec.bin_file(exec_name)
-  end
-
   ##
   # Find the full path to the executable for gem +name+.  If the +exec_name+
   # is not given, an exception will be raised, otherwise the
@@ -315,7 +275,12 @@ module Gem
   # This method should *only* be used in bin stub files.
 
   def self.activate_bin_path(name, exec_name = nil, *requirements) # :nodoc:
-    find_and_activate_spec_for_exe(name, exec_name, requirements).bin_file exec_name
+    spec = find_spec_for_exe name, exec_name, requirements
+    Gem::LOADED_SPECS_MUTEX.synchronize do
+      spec.activate
+      finish_resolve
+    end
+    spec.bin_file exec_name
   end
 
   ##
@@ -328,16 +293,22 @@ module Gem
   ##
   # The path where gem executables are to be installed.
 
-  def self.bindir(install_dir = Gem.dir)
-    return File.join install_dir, "bin" unless
-      install_dir.to_s == Gem.default_dir.to_s
-    Gem.default_bindir
+  def self.bindir(install_dir=Gem.dir, install_root=nil)
+    if install_dir.to_s == Gem.default_dir.to_s
+      install_dir = Gem.default_bindir
+    else
+      install_dir = File.join install_dir, "bin"
+    end
+    unless install_root.nil? or install_root.empty?
+      install_dir = File.join install_root, install_dir
+    end
+    install_dir
   end
 
   ##
   # The path were rubygems plugins are to be installed.
 
-  def self.plugindir(install_dir = Gem.dir)
+  def self.plugindir(install_dir=Gem.dir)
     File.join install_dir, "plugins"
   end
 
@@ -431,23 +402,6 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
     paths.spec_cache_dir
   end
 
-  ##
-  # The RbConfig object for the deployment target platform.
-  #
-  # This is usually the same as the running platform, but may be
-  # different if you are cross-compiling.
-
-  def self.target_rbconfig
-    @target_rbconfig || Gem::TargetRbConfig.for_running_ruby
-  end
-
-  def self.set_target_rbconfig(rbconfig_path)
-    @target_rbconfig = Gem::TargetRbConfig.from_path(rbconfig_path)
-    Gem::Platform.local(refresh: true)
-    Gem.platforms << Gem::Platform.local unless Gem.platforms.include? Gem::Platform.local
-    @target_rbconfig
-  end
-
   ##
   # Quietly ensure the Gem directory +dir+ contains all the proper
   # subdirectories.  If we can't create a directory due to a permission
@@ -502,7 +456,7 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
   # distinction as extensions cannot be shared between the two.
 
   def self.extension_api_version # :nodoc:
-    if target_rbconfig["ENABLE_SHARED"] == "no"
+    if RbConfig::CONFIG["ENABLE_SHARED"] == "no"
       "#{ruby_api_version}-static"
     else
       ruby_api_version
@@ -521,16 +475,16 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
   # Note that find_files will return all files even if they are from different
   # versions of the same gem.  See also find_latest_files
 
-  def self.find_files(glob, check_load_path = true)
+  def self.find_files(glob, check_load_path=true)
     files = []
 
     files = find_files_from_load_path glob if check_load_path
 
     gem_specifications = @gemdeps ? Gem.loaded_specs.values : Gem::Specification.stubs
 
-    files.concat gem_specifications.flat_map {|spec|
+    files.concat gem_specifications.map {|spec|
       spec.matches_for_glob("#{glob}#{Gem.suffix_pattern}")
-    }
+    }.flatten
 
     # $LOAD_PATH might contain duplicate entries or reference
     # the spec dirs directly, so we prune.
@@ -541,9 +495,9 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
 
   def self.find_files_from_load_path(glob) # :nodoc:
     glob_with_suffixes = "#{glob}#{Gem.suffix_pattern}"
-    $LOAD_PATH.flat_map do |load_path|
+    $LOAD_PATH.map do |load_path|
       Gem::Util.glob_files_in_dir(glob_with_suffixes, load_path)
-    end.select {|file| File.file? file }
+    end.flatten.select {|file| File.file? file }
   end
 
   ##
@@ -558,14 +512,14 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
   # Unlike find_files, find_latest_files will return only files from the
   # latest version of a gem.
 
-  def self.find_latest_files(glob, check_load_path = true)
+  def self.find_latest_files(glob, check_load_path=true)
     files = []
 
     files = find_files_from_load_path glob if check_load_path
 
-    files.concat Gem::Specification.latest_specs(true).flat_map {|spec|
+    files.concat Gem::Specification.latest_specs(true).map {|spec|
       spec.matches_for_glob("#{glob}#{Gem.suffix_pattern}")
-    }
+    }.flatten
 
     # $LOAD_PATH might contain duplicate entries or reference
     # the spec dirs directly, so we prune.
@@ -612,7 +566,7 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
       return i if path.instance_variable_defined?(:@gem_prelude_index)
     end
 
-    index = $LOAD_PATH.index RbConfig::CONFIG["sitelibdir"]
+    index = $LOAD_PATH.index RbConfig::CONFIG["vendorlibdir"]
 
     index || 0
   end
@@ -661,30 +615,6 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
     @safe_marshal_loaded = true
   end
 
-  ##
-  # Load Bundler extensions to RubyGems, making sure to avoid redefinition
-  # warnings in platform constants
-
-  def self.load_bundler_extensions(version)
-    return unless version <= Gem::Version.create("2.6.9")
-
-    previous_platforms = {}
-
-    platform_const_list = ["JAVA", "MSWIN", "MSWIN64", "MINGW", "X64_MINGW_LEGACY", "X64_MINGW", "UNIVERSAL_MINGW", "WINDOWS", "X64_LINUX", "X64_LINUX_MUSL"]
-
-    platform_const_list.each do |platform|
-      previous_platforms[platform] = Gem::Platform.const_get(platform)
-      Gem::Platform.send(:remove_const, platform)
-    end
-
-    require "bundler/rubygems_ext"
-
-    platform_const_list.each do |platform|
-      Gem::Platform.send(:remove_const, platform) if Gem::Platform.const_defined?(platform)
-      Gem::Platform.const_set(platform, previous_platforms[platform])
-    end
-  end
-
   ##
   # The file name and line number of the caller of the caller of this method.
   #
@@ -812,6 +742,7 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
     prefix = File.dirname RUBYGEMS_DIR
 
     if prefix != File.expand_path(RbConfig::CONFIG["sitelibdir"]) &&
+       prefix != File.expand_path(RbConfig::CONFIG["vendorlibdir"]) &&
        prefix != File.expand_path(RbConfig::CONFIG["libdir"]) &&
        File.basename(RUBYGEMS_DIR) == "lib"
       prefix
@@ -829,14 +760,18 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
   # Safely read a file in binary mode on all platforms.
 
   def self.read_binary(path)
-    File.binread(path)
+    open_file(path, "rb+", &:read)
+  rescue Errno::EACCES, Errno::EROFS
+    open_file(path, "rb", &:read)
   end
 
   ##
   # Safely write a file in binary mode on all platforms.
 
   def self.write_binary(path, data)
-    File.binwrite(path, data)
+    open_file(path, "wb") do |io|
+      io.write data
+    end
   rescue Errno::ENOSPC
     # If we ran out of space but the file exists, it's *guaranteed* to be corrupted.
     File.delete(path) if File.exist?(path)
@@ -850,38 +785,24 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
     File.open(path, flags, &block)
   end
 
-  ##
-  # Open a file with given flags, and protect access with a file lock
-
-  def self.open_file_with_lock(path, &block)
-    file_lock = "#{path}.lock"
-    open_file_with_flock(file_lock, &block)
-  ensure
-    require "fileutils"
-    FileUtils.rm_f file_lock
-  end
-
   ##
   # Open a file with given flags, and protect access with flock
 
   def self.open_file_with_flock(path, &block)
-    # read-write mode is used rather than read-only in order to support NFS
-    mode = IO::RDWR | IO::APPEND | IO::CREAT | IO::BINARY
-    mode |= IO::SHARE_DELETE if IO.const_defined?(:SHARE_DELETE)
+    flags = File.exist?(path) ? "r+" : "a+"
 
-    File.open(path, mode) do |io|
+    File.open(path, flags) do |io|
       begin
-        # Try to get a lock without blocking.
-        # If we do, the file is locked.
-        # Otherwise, explain why we're waiting and get a lock, but block this time.
-        if io.flock(File::LOCK_EX | File::LOCK_NB) != 0
-          warn "Waiting for another process to let go of lock: #{path}"
-          io.flock(File::LOCK_EX)
-        end
-        io.puts(Process.pid)
+        io.flock(File::LOCK_EX)
       rescue Errno::ENOSYS, Errno::ENOTSUP
       end
       yield io
+    rescue Errno::ENOLCK # NFS
+      if Thread.main != Thread.current
+        raise
+      else
+        open_file(path, flags, &block)
+      end
     end
   end
 
@@ -902,7 +823,7 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
   # Returns a String containing the API compatibility version of Ruby
 
   def self.ruby_api_version
-    @ruby_api_version ||= target_rbconfig["ruby_version"].dup
+    @ruby_api_version ||= RbConfig::CONFIG["ruby_version"].dup
   end
 
   def self.env_requirement(gem_name)
@@ -1078,6 +999,18 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
     self.paths = hash
   end
 
+  ##
+  # Is this a windows platform?
+
+  def self.win_platform?
+    if @@win_platform.nil?
+      ruby_platform = RbConfig::CONFIG["host_os"]
+      @@win_platform = !WIN_PATTERNS.find {|r| ruby_platform =~ r }.nil?
+    end
+
+    @@win_platform
+  end
+
   ##
   # Is this a java platform?
 
@@ -1199,7 +1132,8 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
 
   ##
   # If the SOURCE_DATE_EPOCH environment variable is set, returns it's value.
-  # Otherwise, returns DEFAULT_SOURCE_DATE_EPOCH as a string.
+  # Otherwise, returns the time that +Gem.source_date_epoch_string+ was
+  # first called in the same format as SOURCE_DATE_EPOCH.
   #
   # NOTE(@duckinator): The implementation is a tad weird because we want to:
   #   1. Make builds reproducible by default, by having this function always
@@ -1214,12 +1148,15 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
   # https://reproducible-builds.org/specs/source-date-epoch/
 
   def self.source_date_epoch_string
+    # The value used if $SOURCE_DATE_EPOCH is not set.
+    @default_source_date_epoch ||= Time.now.to_i.to_s
+
     specified_epoch = ENV["SOURCE_DATE_EPOCH"]
 
     # If it's empty or just whitespace, treat it like it wasn't set at all.
     specified_epoch = nil if !specified_epoch.nil? && specified_epoch.strip.empty?
 
-    epoch = specified_epoch || DEFAULT_SOURCE_DATE_EPOCH.to_s
+    epoch = specified_epoch || @default_source_date_epoch
 
     epoch.strip
   end
@@ -1244,13 +1181,6 @@ An Array (#{env.inspect}) was passed in from #{caller[3]}
 
     attr_accessor :disable_system_update_message
 
-    unless ENV['RUBYGEMS_SYSTEM_UPDATE_MESSAGE'] == 'enable'
-      Gem.disable_system_update_message = <<EOF
-Ruby was installed using an OpenBSD package, and using `gem update --system`
-is not supported.
-EOF
-    end
-
     ##
     # Whether RubyGems should enhance builtin `require` to automatically
     # check whether the path required is present in installed gems, and
@@ -1441,6 +1371,17 @@ begin
 rescue LoadError
 end
 
+# TruffleRuby >= 24 defines REUSE_AS_BINARY_ON_TRUFFLERUBY in defaults/truffleruby.
+# However, TruffleRuby < 24 defines REUSE_AS_BINARY_ON_TRUFFLERUBY directly in its copy
+# of lib/rubygems/platform.rb, so it is not defined if RubyGems is updated (gem update --system).
+# Instead, we define it here in that case, similar to bundler/lib/bundler/rubygems_ext.rb.
+# We must define it here and not in platform.rb because platform.rb is loaded before defaults/truffleruby.
+class Gem::Platform
+  if RUBY_ENGINE == "truffleruby" && !defined?(REUSE_AS_BINARY_ON_TRUFFLERUBY)
+    REUSE_AS_BINARY_ON_TRUFFLERUBY = %w[libv8 libv8-node sorbet-static].freeze
+  end
+end
+
 ##
 # Loads the default specs.
 Gem::Specification.load_defaults
