@@ -76,15 +76,21 @@ class Gem::BasicSpecification
     elsif missing_extensions?
       @ignored = true
 
-      if Gem::Platform::RUBY == platform || Gem::Platform.local === platform
-        warn "Ignoring #{full_name} because its extensions are not built. " +
+      if platform == Gem::Platform::RUBY || Gem::Platform.local === platform
+        warn "Ignoring #{full_name} because its extensions are not built. " \
              "Try: gem pristine #{name} --version #{version}"
       end
 
       return false
     end
 
-    have_file? file, Gem.suffixes
+    is_soext = file.end_with?(".so", ".o")
+
+    if is_soext
+      have_file? file.delete_suffix(File.extname(file)), Gem.dynamic_library_suffixes
+    else
+      have_file? file, Gem.suffixes
+    end
   end
 
   def default_gem?
@@ -92,11 +98,25 @@ class Gem::BasicSpecification
       File.dirname(loaded_from) == Gem.default_specifications_dir
   end
 
+  ##
+  # Regular gems take precedence over default gems
+
+  def default_gem_priority
+    default_gem? ? 1 : -1
+  end
+
+  ##
+  # Gems higher up in +gem_path+ take precedence
+
+  def base_dir_priority(gem_path)
+    gem_path.index(base_dir) || gem_path.size
+  end
+
   ##
   # Returns full path to the directory where gem's extensions are installed.
 
   def extension_dir
-    @extension_dir ||= File.expand_path(File.join(extensions_dir, full_name)).tap(&Gem::UNTAINT)
+    @extension_dir ||= File.expand_path(File.join(extensions_dir, full_name))
   end
 
   ##
@@ -110,9 +130,7 @@ class Gem::BasicSpecification
 
   def find_full_gem_path # :nodoc:
     # TODO: also, shouldn't it default to full_name if it hasn't been written?
-    path = File.expand_path File.join(gems_dir, full_name)
-    path.tap(&Gem::UNTAINT)
-    path
+    File.expand_path File.join(gems_dir, full_name)
   end
 
   private :find_full_gem_path
@@ -133,9 +151,22 @@ class Gem::BasicSpecification
 
   def full_name
     if platform == Gem::Platform::RUBY || platform.nil?
-      "#{name}-#{version}".dup.tap(&Gem::UNTAINT)
+      "#{name}-#{version}"
     else
-      "#{name}-#{version}-#{platform}".dup.tap(&Gem::UNTAINT)
+      "#{name}-#{version}-#{platform}"
+    end
+  end
+
+  ##
+  # Returns the full name of this Gem (see `Gem::BasicSpecification#full_name`).
+  # Information about where the gem is installed is also included if not
+  # installed in the default GEM_HOME.
+
+  def full_name_with_location
+    if base_dir != Gem.dir
+      "#{full_name} in #{base_dir}"
+    else
+      full_name
     end
   end
 
@@ -145,15 +176,15 @@ class Gem::BasicSpecification
 
   def full_require_paths
     @full_require_paths ||=
-    begin
-      full_paths = raw_require_paths.map do |path|
-        File.join full_gem_path, path.tap(&Gem::UNTAINT)
+      begin
+        full_paths = raw_require_paths.map do |path|
+          File.join full_gem_path, path
+        end
+
+        full_paths << extension_dir if have_extensions?
+
+        full_paths
       end
-
-      full_paths << extension_dir if have_extensions?
-
-      full_paths
-    end
   end
 
   ##
@@ -161,7 +192,7 @@ class Gem::BasicSpecification
 
   def datadir
     # TODO: drop the extra ", gem_name" which is uselessly redundant
-    File.expand_path(File.join(gems_dir, full_name, "data", name)).tap(&Gem::UNTAINT)
+    File.expand_path(File.join(gems_dir, full_name, "data", name))
   end
 
   ##
@@ -171,18 +202,14 @@ class Gem::BasicSpecification
   def to_fullpath(path)
     if activated?
       @paths_map ||= {}
-      @paths_map[path] ||=
-      begin
-        fullpath = nil
-        suffixes = Gem.suffixes
-        suffixes.find do |suf|
-          full_require_paths.find do |dir|
-            File.file?(fullpath = "#{dir}/#{path}#{suf}")
-          end
-        end ? fullpath : nil
+      Gem.suffixes.each do |suf|
+        full_require_paths.each do |dir|
+          fullpath = "#{dir}/#{path}#{suf}"
+          next unless File.file?(fullpath)
+          @paths_map[path] ||= fullpath
+        end
       end
-    else
-      nil
+      @paths_map[path]
     end
   end
 
@@ -272,9 +299,9 @@ class Gem::BasicSpecification
   # Return all files in this gem that match for +glob+.
 
   def matches_for_glob(glob) # TODO: rename?
-    glob = File.join(self.lib_dirs_glob, glob)
+    glob = File.join(lib_dirs_glob, glob)
 
-    Dir[glob].map {|f| f.tap(&Gem::UNTAINT) } # FIX our tests are broken, run w/ SAFE=1
+    Dir[glob]
   end
 
   ##
@@ -289,17 +316,17 @@ class Gem::BasicSpecification
   # for this spec.
 
   def lib_dirs_glob
-    dirs = if self.raw_require_paths
-      if self.raw_require_paths.size > 1
-        "{#{self.raw_require_paths.join(',')}}"
+    dirs = if raw_require_paths
+      if raw_require_paths.size > 1
+        "{#{raw_require_paths.join(",")}}"
       else
-        self.raw_require_paths.first
+        raw_require_paths.first
       end
     else
       "lib" # default value for require_paths for bundler/inline
     end
 
-    "#{self.full_gem_path}/#{dirs}".dup.tap(&Gem::UNTAINT)
+    "#{full_gem_path}/#{dirs}"
   end
 
   ##
@@ -324,15 +351,19 @@ class Gem::BasicSpecification
     raise NotImplementedError
   end
 
-  def this; self; end
+  def this
+    self
+  end
 
   private
 
-  def have_extensions?; !extensions.empty?; end
+  def have_extensions?
+    !extensions.empty?
+  end
 
   def have_file?(file, suffixes)
     return true if raw_require_paths.any? do |path|
-      base = File.join(gems_dir, full_name, path.tap(&Gem::UNTAINT), file).tap(&Gem::UNTAINT)
+      base = File.join(gems_dir, full_name, path, file)
       suffixes.any? {|suf| File.file? base + suf }
     end
 
