@@ -107,6 +107,15 @@ class Gem::Commands::SetupCommand < Gem::Command
     @verbose = nil
   end
 
+  def check_ruby_version
+    required_version = Gem::Requirement.new ">= 2.6.0"
+
+    unless required_version.satisfied_by? Gem.ruby_version
+      alert_error "Expected Ruby version #{required_version}, is #{Gem.ruby_version}"
+      terminate_interaction 1
+    end
+  end
+
   def defaults_str # :nodoc:
     "--format-executable --document ri --regenerate-binstubs"
   end
@@ -139,6 +148,8 @@ By default, this RubyGems will install gem as:
   def execute
     @verbose = Gem.configuration.really_verbose
 
+    check_ruby_version
+
     require "fileutils"
     if Gem.configuration.really_verbose
       extend FileUtils::Verbose
@@ -175,7 +186,7 @@ By default, this RubyGems will install gem as:
 
     uninstall_old_gemcutter
 
-    documentation_success = install_rdoc
+    documentation_success = install_rdoc install_destdir
 
     say
     if @verbose
@@ -268,7 +279,11 @@ By default, this RubyGems will install gem as:
           File.open bin_cmd_file, "w" do |file|
             file.puts <<-TEXT
   @ECHO OFF
-  @"%~dp0#{File.basename(Gem.ruby).chomp('"')}" "%~dpn0" %*
+  IF NOT "%~f0" == "~f0" GOTO :WinNT
+  @"#{File.basename(Gem.ruby).chomp('"')}" "#{dest_file}" %1 %2 %3 %4 %5 %6 %7 %8 %9
+  GOTO :EOF
+  :WinNT
+  @"#{File.basename(Gem.ruby).chomp('"')}" "%~dpn0" %*
   TEXT
           end
 
@@ -304,11 +319,15 @@ By default, this RubyGems will install gem as:
     end
   end
 
-  def install_rdoc
+  def install_rdoc(install_destdir)
     gem_doc_dir = File.join Gem.dir, "doc"
     rubygems_name = "rubygems-#{Gem::VERSION}"
     rubygems_doc_dir = File.join gem_doc_dir, rubygems_name
 
+    unless install_destdir.empty?
+      FileUtils.mkdir_p gem_doc_dir unless File.exist?(gem_doc_dir)
+    end
+
     begin
       Gem.ensure_gem_subdirectories Gem.dir
     rescue SystemCallError
@@ -319,14 +338,12 @@ By default, this RubyGems will install gem as:
        (!File.exist?(rubygems_doc_dir) ||
         File.writable?(rubygems_doc_dir))
       say "Removing old RubyGems RDoc and ri" if @verbose
-      Dir[File.join(Gem.dir, "doc", "rubygems-[0-9]*")].each do |dir|
+      Dir[File.join(gem_doc_dir, "rubygems-[0-9]*")].each do |dir|
         rm_rf dir
       end
 
       require_relative "../rdoc"
 
-      return false unless defined?(Gem::RDoc)
-
       fake_spec = Gem::Specification.new "rubygems", Gem::VERSION
       def fake_spec.full_gem_path
         File.expand_path "../../..", __dir__
@@ -350,15 +367,9 @@ By default, this RubyGems will install gem as:
   def install_default_bundler_gem(bin_dir)
     current_default_spec = Gem::Specification.default_stubs.find {|s| s.name == "bundler" }
     specs_dir = if current_default_spec && default_dir == Gem.default_dir
-      all_specs_current_version = Gem::Specification.stubs.select {|s| s.full_name == current_default_spec.full_name }
-
       Gem::Specification.remove_spec current_default_spec
       loaded_from = current_default_spec.loaded_from
       File.delete(loaded_from)
-
-      # Remove previous default gem executables if they were not shadowed by a regular gem
-      FileUtils.rm_rf current_default_spec.full_gem_path if all_specs_current_version.size == 1
-
       File.dirname(loaded_from)
     else
       target_specs_dir = File.join(default_dir, "specifications", "default")
