@@ -8,6 +8,7 @@
 
 require_relative "installer_uninstaller_utils"
 require_relative "exceptions"
+require_relative "deprecate"
 require_relative "package"
 require_relative "ext"
 require_relative "user_interaction"
@@ -26,6 +27,8 @@ require_relative "user_interaction"
 # file.  See Gem.pre_install and Gem.post_install for details.
 
 class Gem::Installer
+  extend Gem::Deprecate
+
   ##
   # Paths where env(1) might live.  Some systems are broken and have it in
   # /bin
@@ -63,7 +66,31 @@ class Gem::Installer
 
   attr_reader :package
 
+  @path_warning = false
+
   class << self
+    #
+    # Changes in rubygems to lazily loading `rubygems/command` (in order to
+    # lazily load `optparse` as a side effect) affect bundler's custom installer
+    # which uses `Gem::Command` without requiring it (up until bundler 2.2.29).
+    # This hook is to compensate for that missing require.
+    #
+    # TODO: Remove when rubygems no longer supports running on bundler older
+    # than 2.2.29.
+
+    def inherited(klass)
+      if klass.name == "Bundler::RubyGemsGemInstaller"
+        require "rubygems/command"
+      end
+
+      super(klass)
+    end
+
+    ##
+    # True if we've warned about PATH not including Gem.bindir
+
+    attr_accessor :path_warning
+
     ##
     # Overrides the executable format.
     #
@@ -140,6 +167,9 @@ class Gem::Installer
   #                      foo_exec18.
   # :ignore_dependencies:: Don't raise if a dependency is missing.
   # :install_dir:: The directory to install the gem into.
+  # :install_root:: The directory to use as a buildroot for "destdir"-style
+  #		  installation.  All paths during installation are relative
+  #               to the buildroot.
   # :security_policy:: Use the specified security policy.  See Gem::Security
   # :user_install:: Indicate that the gem should be unpacked into the users
   #                 personal gem directory.
@@ -150,7 +180,7 @@ class Gem::Installer
   #               process. If not set, then Gem::Command.build_args is used
   # :post_install_message:: Print gem post install message if true
 
-  def initialize(package, options = {})
+  def initialize(package, options={})
     require "fileutils"
 
     @options = options
@@ -161,6 +191,15 @@ class Gem::Installer
     @package.dir_mode = options[:dir_mode]
     @package.prog_mode = options[:prog_mode]
     @package.data_mode = options[:data_mode]
+
+    if @gem_home == Gem.user_dir
+      # If we get here, then one of the following likely happened:
+      # - `--user-install` was specified
+      # - `Gem::PathSupport#home` fell back to `Gem.user_dir`
+      # - GEM_HOME was manually set to `Gem.user_dir`
+
+      check_that_user_bin_dir_is_in_path
+    end
   end
 
   ##
@@ -208,7 +247,8 @@ class Gem::Installer
       ruby_executable = true
       existing = io.read.slice(/
           ^\s*(
-            Gem\.activate_and_load_bin_path\( |
+            gem \s |
+            load \s Gem\.bin_path\( |
             load \s Gem\.activate_bin_path\(
           )
           (['"])(.*?)(\2),
@@ -271,7 +311,11 @@ class Gem::Installer
     run_pre_install_hooks
 
     # Set loaded_from to ensure extension_dir is correct
-    spec.loaded_from = spec_file
+    if @options[:install_as_default]
+      spec.loaded_from = default_spec_file
+    else
+      spec.loaded_from = spec_file
+    end
 
     # Completely remove any previous gem files
     FileUtils.rm_rf gem_dir
@@ -280,17 +324,24 @@ class Gem::Installer
     dir_mode = options[:dir_mode]
     FileUtils.mkdir_p gem_dir, mode: dir_mode && 0o755
 
-    extract_files
+    if @options[:install_as_default]
+      extract_bin
+      write_default_spec
+    else
+      extract_files
 
-    build_extensions
-    write_build_info_file
-    run_post_build_hooks
+      build_extensions
+      write_build_info_file
+      run_post_build_hooks
+    end
 
     generate_bin
     generate_plugins
 
-    write_spec
-    write_cache_file
+    unless @options[:install_as_default]
+      write_spec
+      write_cache_file
+    end
 
     File.chmod(dir_mode, gem_dir) if dir_mode
 
@@ -378,6 +429,15 @@ class Gem::Installer
     !dependency.matching_specs.empty?
   end
 
+  ##
+  # Unpacks the gem into the given directory.
+
+  def unpack(directory)
+    @gem_dir = directory
+    extract_files
+  end
+  rubygems_deprecate :unpack
+
   ##
   # The location of the spec file that is installed.
   #
@@ -386,18 +446,12 @@ class Gem::Installer
     File.join gem_home, "specifications", "#{spec.full_name}.gemspec"
   end
 
-  def default_spec_dir
-    dir = File.join(gem_home, "specifications", "default")
-    FileUtils.mkdir_p dir
-    dir
-  end
-
   ##
   # The location of the default spec file for default gems.
   #
 
   def default_spec_file
-    File.join default_spec_dir, "#{spec.full_name}.gemspec"
+    File.join gem_home, "specifications", "default", "#{spec.full_name}.gemspec"
   end
 
   ##
@@ -437,21 +491,11 @@ class Gem::Installer
   end
 
   def generate_bin # :nodoc:
-    executables = spec.executables
-    return if executables.nil? || executables.empty?
-
-    if @gem_home == Gem.user_dir
-      # If we get here, then one of the following likely happened:
-      # - `--user-install` was specified
-      # - `Gem::PathSupport#home` fell back to `Gem.user_dir`
-      # - GEM_HOME was manually set to `Gem.user_dir`
-
-      check_that_user_bin_dir_is_in_path(executables)
-    end
+    return if spec.executables.nil? || spec.executables.empty?
 
     ensure_writable_dir @bin_dir
 
-    executables.each do |filename|
+    spec.executables.each do |filename|
       bin_path = File.join gem_dir, spec.bindir, filename
       next unless File.exist? bin_path
 
@@ -497,7 +541,7 @@ class Gem::Installer
   def generate_bin_script(filename, bindir)
     bin_script_path = File.join bindir, formatted_program_filename(filename)
 
-    Gem.open_file_with_lock(bin_script_path) do
+    Gem.open_file_with_flock("#{bin_script_path}.lock") do |lock|
       require "fileutils"
       FileUtils.rm_f bin_script_path # prior install may have been --no-wrappers
 
@@ -505,6 +549,8 @@ class Gem::Installer
         file.write app_script_text(filename)
         file.chmod(options[:prog_mode] || 0o755)
       end
+    ensure
+      FileUtils.rm_f lock.path
     end
 
     verbose bin_script_path
@@ -635,14 +681,21 @@ class Gem::Installer
     @build_root          = options[:build_root]
 
     @build_args = options[:build_args]
-    @build_jobs = options[:build_jobs]
+
+    install_root         = options[:install_root]
 
     @gem_home = @install_dir || user_install_dir || Gem.dir
 
     # If the user has asked for the gem to be installed in a directory that is
     # the system gem directory, then use the system bin directory, else create
     # (or use) a new bin dir under the gem_home.
-    @bin_dir ||= Gem.bindir(@gem_home)
+    @bin_dir ||= Gem.bindir(@gem_home, install_root)
+
+    unless install_root.nil? or install_root.empty?
+      @install_root = File.expand_path install_root
+      @gem_home = File.join(@install_root, @gem_home)
+      @plugins_dir         = Gem.plugindir(@gem_home)
+    end
 
     @plugins_dir = Gem.plugindir(@gem_home)
 
@@ -654,7 +707,9 @@ class Gem::Installer
     end
   end
 
-  def check_that_user_bin_dir_is_in_path(executables) # :nodoc:
+  def check_that_user_bin_dir_is_in_path # :nodoc:
+    return if self.class.path_warning
+
     user_bin_dir = @bin_dir || Gem.bindir(gem_home)
     user_bin_dir = user_bin_dir.tr(File::ALT_SEPARATOR, File::SEPARATOR) if File::ALT_SEPARATOR
 
@@ -670,7 +725,8 @@ class Gem::Installer
 
     unless path.include? user_bin_dir
       unless !Gem.win_platform? && (path.include? user_bin_dir.sub(ENV["HOME"], "~"))
-        alert_warning "You don't have #{user_bin_dir} in your PATH,\n\t  gem executables (#{executables.join(", ")}) will not run."
+        alert_warning "You don't have #{user_bin_dir} in your PATH,\n\t  gem executables will not run."
+        self.class.path_warning = true
       end
     end
   end
@@ -715,53 +771,54 @@ class Gem::Installer
   def app_script_text(bin_file_name)
     # NOTE: that the `load` lines cannot be indented, as old RG versions match
     # against the beginning of the line
-    <<~TEXT
-      #{shebang bin_file_name}
-      #
-      # This file was generated by RubyGems.
-      #
-      # The application '#{spec.name}' is installed as part of a gem, and
-      # this file is here to facilitate running it.
-      #
+    <<-TEXT
+#{shebang bin_file_name}
+#
+# This file was generated by RubyGems.
+#
+# The application '#{spec.name}' is installed as part of a gem, and
+# this file is here to facilitate running it.
+#
 
-      require 'rubygems'
-      #{gemdeps_load(spec.name)}
-      version = "#{Gem::Requirement.default_prerelease}"
+require 'rubygems'
+#{gemdeps_load(spec.name)}
+version = "#{Gem::Requirement.default_prerelease}"
 
-      str = ARGV.first
-      if str
-        str = str.b[/\\A_(.*)_\\z/, 1]
-        if str and Gem::Version.correct?(str)
-          #{explicit_version_requirement(spec.name)}
-          ARGV.shift
-        end
-      end
+str = ARGV.first
+if str
+  str = str.b[/\\A_(.*)_\\z/, 1]
+  if str and Gem::Version.correct?(str)
+    #{explicit_version_requirement(spec.name)}
+    ARGV.shift
+  end
+end
 
-      if Gem.respond_to?(:activate_and_load_bin_path)
-        Gem.activate_and_load_bin_path('#{spec.name}', '#{bin_file_name}', version)
-      else
-        load Gem.activate_bin_path('#{spec.name}', '#{bin_file_name}', version)
-      end
-    TEXT
+if Gem.respond_to?(:activate_bin_path)
+load Gem.activate_bin_path('#{spec.name}', '#{bin_file_name}', version)
+else
+gem #{spec.name.dump}, version
+load Gem.bin_path(#{spec.name.dump}, #{bin_file_name.dump}, version)
+end
+TEXT
   end
 
   def gemdeps_load(name)
     return "" if name == "bundler"
 
-    <<~TEXT
+    <<-TEXT
 
-      Gem.use_gemdeps
-    TEXT
+Gem.use_gemdeps
+TEXT
   end
 
   def explicit_version_requirement(name)
     code = "version = str"
     return code unless name == "bundler"
 
-    code += <<~TEXT
+    code += <<-TEXT
 
-      ENV['BUNDLER_VERSION'] = str
-    TEXT
+    ENV['BUNDLER_VERSION'] = str
+TEXT
   end
 
   ##
@@ -776,9 +833,9 @@ class Gem::Installer
 
     if File.exist?(File.join(bindir, ruby_exe))
       # stub & ruby.exe within same folder.  Portable
-      <<~TEXT
-        @ECHO OFF
-        @"%~dp0#{ruby_exe}" "%~dpn0" %*
+      <<-TEXT
+@ECHO OFF
+@"%~dp0#{ruby_exe}" "%~dpn0" %*
       TEXT
     elsif bindir.downcase.start_with? rb_topdir.downcase
       # stub within ruby folder, but not standard bin.  Portable
@@ -786,16 +843,16 @@ class Gem::Installer
       from = Pathname.new bindir
       to   = Pathname.new "#{rb_topdir}/bin"
       rel  = to.relative_path_from from
-      <<~TEXT
-        @ECHO OFF
-        @"%~dp0#{rel}/#{ruby_exe}" "%~dpn0" %*
+      <<-TEXT
+@ECHO OFF
+@"%~dp0#{rel}/#{ruby_exe}" "%~dpn0" %*
       TEXT
     else
       # outside ruby folder, maybe -user-install or bundler.  Portable, but ruby
       # is dependent on PATH
-      <<~TEXT
-        @ECHO OFF
-        @#{ruby_exe} "%~dpn0" %*
+      <<-TEXT
+@ECHO OFF
+@#{ruby_exe} "%~dpn0" %*
       TEXT
     end
   end
@@ -804,7 +861,7 @@ class Gem::Installer
   # configure scripts and rakefiles or mkrf_conf files.
 
   def build_extensions
-    builder = Gem::Ext::Builder.new spec, build_args, Gem.target_rbconfig, build_jobs
+    builder = Gem::Ext::Builder.new spec, build_args
 
     builder.build_extensions
   end
@@ -873,7 +930,11 @@ class Gem::Installer
 
     ensure_loadable_spec
 
-    Gem.ensure_gem_subdirectories gem_home
+    if options[:install_as_default]
+      Gem.ensure_default_gem_subdirectories gem_home
+    else
+      Gem.ensure_gem_subdirectories gem_home
+    end
 
     return true if @force
 
@@ -890,6 +951,9 @@ class Gem::Installer
     return if build_args.empty?
 
     build_info_dir = File.join gem_home, "build_info"
+    unless @install_root.nil? or @install_root.empty?
+      build_info_dir = File.join @gem_home, "build_info"
+    end
 
     dir_mode = options[:dir_mode]
     FileUtils.mkdir_p build_info_dir, mode: dir_mode && 0o755
@@ -914,8 +978,11 @@ class Gem::Installer
   end
 
   def ensure_writable_dir(dir) # :nodoc:
-    require "fileutils"
-    FileUtils.mkdir_p dir, mode: options[:dir_mode] && 0o755
+    begin
+      Dir.mkdir dir, *[options[:dir_mode] && 0o755].compact
+    rescue SystemCallError
+      raise unless File.directory? dir
+    end
 
     raise Gem::FilePermissionError.new(dir) unless File.writable? dir
   end
@@ -942,12 +1009,8 @@ class Gem::Installer
                     end
   end
 
-  def build_jobs
-    @build_jobs ||= Etc.nprocessors + 1
-  end
-
   def rb_config
-    Gem.target_rbconfig
+    RbConfig::CONFIG
   end
 
   def ruby_install_name
@@ -960,17 +1023,18 @@ class Gem::Installer
 
   def bash_prolog_script
     if load_relative_enabled?
+      script = +<<~EOS
+        bindir="${0%/*}"
+      EOS
+
+      script << %(exec "$bindir/#{ruby_install_name}" "-x" "$0" "$@"\n)
+
       <<~EOS
         #!/bin/sh
         # -*- ruby -*-
         _=_\\
         =begin
-        bindir="${0%/*}"
-        ruby="$bindir/#{ruby_install_name}"
-        if [ ! -f "$ruby" ]; then
-          ruby="#{ruby_install_name}"
-        fi
-        exec "$ruby" "-x" "$0" "$@"
+        #{script.chomp}
         =end
       EOS
     else
