@@ -30,7 +30,7 @@ module Gem
     # Returns the list of all specifications in the record
 
     def all
-      @all ||= stubs.map(&:to_spec)
+      @all ||= Gem.loaded_specs.values | stubs.map(&:to_spec)
     end
 
     ##
@@ -73,7 +73,7 @@ module Gem
     end
 
     ##
-    # Adds +spec+ to the record, keeping the collection properly sorted.
+    # Adds +spec+ to the the record, keeping the collection properly sorted.
 
     def add_spec(spec)
       return if all.include? spec
