@@ -125,31 +125,6 @@ module Test
       AVAILABLE_ORDERS = [:alphabetic, :random, :defined] # :nodoc:
 
       class << self
-        # Indicates whether the test is parallel safe.
-        #
-        # Tests that this method returns `false` are executed sequentially
-        # before parallel safe tests run. This only works when the `--parallel`
-        # option is specified.
-        #
-        # @example Indicates that test_parallel_unsafe is parallel unsafe
-        #
-        #   class TestMyClass < Test::Unit::TestCase
-        #     class << self
-        #       def parallel_safe?
-        #         false
-        #       end
-        #     end
-        #
-        #     def test_parallel_unsafe
-        #       # ...
-        #     end
-        #   end
-        #
-        # @since 3.6.3
-        def parallel_safe?
-          true
-        end
-
         def inherited(sub_class) # :nodoc:
           DESCENDANTS << sub_class
           super
@@ -577,7 +552,7 @@ module Test
       # Runs the individual test method represented by this
       # instance of the fixture, collecting statistics, failures
       # and errors in result.
-      def run(result, runner_class: nil)
+      def run(result)
         begin
           @_result = result
           @internal_data.test_started
